/*
 * imgInit.c
 * Dan Wallach <dwallach@cs.princeton.edu>
 * 26 September 1995
 *
 * $Id: imgInit.c,v 1.4 1995/09/30 21:10:25 dwallach Exp $
 */

#include <tcl.h>
#include "img.h" 



typedef struct {
    char* name;
    int (*FromFunc)(Img*);
    Img* (*ToFunc)(double value);
} ContinuousFilter;

typedef struct {
    char* name;
    char* argType;
    int (*FromFunc)(Img*);
    Img* (*ToFunc)(char* value);
} EnumeratedFilter;

/***************************************************************************
 * If you're adding a new filter, add it here, and in tkenhance
 ***************************************************************************/
 
static ContinuousFilter cfArray[] = {
    {"img:randomnoise", Img_RandomNoiseFrom, Img_RandomNoiseTo},
    {"img:gamma", Img_GammaFrom, Img_GammaTo},
    {"img:saturation", Img_SaturationFrom, Img_SaturationTo},
    {"img:contrast", Img_ContrastFrom, Img_ContrastTo},
    {"img:brightness", Img_BrightnessFrom, Img_BrightnessTo},
    {"img:sharpen", Img_SharpenFrom, Img_SharpenTo},
    {NULL, NULL, NULL}
};

static EnumeratedFilter efArray[] = {
    {"img:quantization", "-nbits", Img_QuantizationFrom, Img_QuantizationTo},
    {"img:dither", "-type", Img_DitherFrom, Img_DitherTo},
    {"img:fun1", "-type", Img_Fun1From, Img_Fun1To},
    {NULL, NULL, NULL}
};

/***************************************************************************
 * You shouldn't need to change anything else in this file
 ***************************************************************************/


static int
Img_Continuous(ClientData cd, Tcl_Interp* interp, int argc, char** argv) {
    ContinuousFilter* pcf = (ContinuousFilter*)cd;
    Img* img;
    double dvalue;
    
    if(argc < 3) {
        Tcl_AppendResult(interp, "wrong # args, shoud be: \"", argv[0],
			 " from image\" or \"to image -val value\"", NULL);
	return TCL_ERROR;
    }
    
    if(argv[1][0] == 'f' && !strcmp(argv[1], "from")) {
        if(argc != 3) {
	    Tcl_AppendResult(interp, "wrong # args, shoud be: \"", argv[0],
			     " from image\" or \"to image -val value\"", NULL);
	    return TCL_ERROR;
	}
	
	if((img = Img_New(argv[2])) == NULL) {
	    Tcl_AppendResult(interp, "error reading image \"", argv[2], "\"", NULL);
	    return TCL_ERROR;
	}
	
	if(pcf->FromFunc(img) != 0) {
	    Tcl_AppendResult(interp, "error running filter", NULL);
	    return TCL_ERROR;
	}
	
	return TCL_OK;
    }
    
    if(argv[1][0] == 't' && !strcmp(argv[1], "to")) {
        if(argc != 5 || strcmp(argv[3], "-val")) {
	    Tcl_AppendResult(interp, "wrong # args, shoud be: \"", argv[0],
			     " from image\" or \"to image -val value\"", NULL);
	    return TCL_ERROR;
	}
	
	if(Tcl_GetDouble(interp, argv[4], &dvalue) != TCL_OK)
	    return TCL_ERROR;
	if((img = pcf->ToFunc(dvalue)) == NULL) {
	    Tcl_AppendResult(interp, "error running filter", NULL);
	    return TCL_ERROR;
	}
	
	if(Img_PutTk(img, argv[2]) != 0) {
	    Tcl_AppendResult(interp, "error copying image to Tk", NULL);
	    return TCL_ERROR;
	}
	
	return TCL_OK;
    }
    
    Tcl_AppendResult(interp, "wrong # args, shoud be: \"", argv[0],
		     " from image\" or \"to image -val value\"", NULL);

    return TCL_ERROR;
}

static int
Img_Enumerated(ClientData cd, Tcl_Interp* interp, int argc, char** argv) {
    EnumeratedFilter* pef = (EnumeratedFilter*)cd;
    Img* img;
    
    if(argc < 3) {
        Tcl_AppendResult(interp, "wrong # args, shoud be: \"", argv[0],
			 " from image\" or \"to image ", pef->argType,
			 " value\"", NULL);
	return TCL_ERROR;
    }
    
    if(argv[1][0] == 'f' && !strcmp(argv[1], "from")) {
        if(argc != 3) {
	    Tcl_AppendResult(interp, "wrong # args, shoud be: \"", argv[0],
			     " from image\" or \"to image ", pef->argType,
			     " value\"", NULL);
	    return TCL_ERROR;
	}
	
	if((img = Img_New(argv[2])) == NULL) {
	    Tcl_AppendResult(interp, "error reading image \"", argv[2], "\"", NULL);
	    return TCL_ERROR;
	}
	
	if(pef->FromFunc(img) != 0) {
	    Tcl_AppendResult(interp, "error running filter", NULL);
	    return TCL_ERROR;
	}
	
	return TCL_OK;
    }
    
    if(argv[1][0] == 't' && !strcmp(argv[1], "to")) {
        if(argc != 5 || strcmp(argv[3], pef->argType)) {
	    Tcl_AppendResult(interp, "wrong # args, shoud be: \"", argv[0],
			     " from image\" or \"to image ", pef->argType,
			     " value\"", NULL);
	    return TCL_ERROR;
	}
	
	if((img = pef->ToFunc(argv[4])) == NULL) {
	    Tcl_AppendResult(interp, "error running filter", NULL);
	    return TCL_ERROR;
	}
	
	if(Img_PutTk(img, argv[2]) != 0) {
	    Tcl_AppendResult(interp, "error copying image to Tk", NULL);
	    return TCL_ERROR;
	}
	
	return TCL_OK;
    }
    
    Tcl_AppendResult(interp, "wrong # args, shoud be: \"", argv[0],
		     " from image\" or \"to image ", pef->argType,
		     " value\"", NULL);

    return TCL_ERROR;
}


int
Img_Init(Tcl_Interp* interp) {
    ContinuousFilter* pcf;
    EnumeratedFilter* pef;
    
    for(pcf=cfArray; pcf->FromFunc != NULL; pcf++) {
	Tcl_CreateCommand(interp, pcf->name, Img_Continuous,
			  (ClientData)pcf, NULL);
    }
    
    for(pef=efArray; pef->FromFunc != NULL; pef++) {
	Tcl_CreateCommand(interp, pef->name, Img_Enumerated,
			  (ClientData)pef, NULL);
    }
}
